#include <qapplication.h>
#ifdef QTOPIA
#include <qpe/config.h>
#else
#include <config.h>
#endif

#include <qfile.h>
#include <qtextstream.h>
#include <kmdcodec.h>
#include <debug.h>

#include "configuration.h"

void saveWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName)
{
#ifndef QTOPIA
	Config cf(configName, Config::File);
	cf.setGroup(groupName);
	
	bool wasMaximized = widget->isMaximized();
	
	cf.writeEntry("isMaximized", wasMaximized);
	
	if (wasMaximized)
		widget->showNormal();
	
	cf.writeEntry("x", widget->x());
	cf.writeEntry("y", widget->y());
	
	cf.writeEntry("width", widget->size().width());
	cf.writeEntry("height", widget->size().height());

	if (wasMaximized)
		widget->showMaximized();
#endif
}

void loadWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName, int defX, int defY, int defWidth, int defHeight)
{
#ifndef QTOPIA
	Config cf(configName, Config::File);
	cf.setGroup(groupName);
	
	QSize size(cf.readNumEntry("width", defWidth), cf.readNumEntry("height", defHeight));
	QPoint point(cf.readNumEntry("x", defX), cf.readNumEntry("y", defY));
	
	// Use a two step approach to avoid flickering and resizing and
	// work around an issue with some WMs which causes the windows position
	// to be wrongly reported if we save later on and the user didn't change
	// the position or size of the frame.
	
	// first resize and positioning so Qt knows what we are up to
	widget->resize(size);
	widget->move(point);
	// show window, so the WM is prepared...
	widget->showNormal();
	// finally set again, so the WM explicitly gets it...
	widget->resize(size);
	widget->move(point);
	
	if (cf.readBoolEntry("isMaximized", false))
		widget->showMaximized();
#endif
}


Configuration & Configuration::singleton()
{
	static Configuration instance;
	return instance;
}

Configuration::Configuration()
{
    username_ = "";
    md5Password_ = "";
    cacheSaveInterval_ = 20;
    submissionPaused_ = false;
}

Configuration::~Configuration()
{
	write();
}

bool Configuration::isValidConfig() const
{
	Config cf(getConfigName(), Config::File);
	cf.setGroup(CONFIGGROUPNAME);

	bool isValid = false;
	if (cf.readNumEntry("CONFIG_VERSION", -1) == configVersion)
		isValid = true;

	return isValid;
}

void Configuration::write() const
{
	Config cf(getConfigName(), Config::File);

	cf.setGroup(CONFIGGROUPNAME);
	cf.writeEntry("CONFIG_VERSION", configVersion);
	
	cf.writeEntry("username", username_);
	cf.writeEntry("password", md5Password_);
	cf.writeEntry("cacheSaveInterval", cacheSaveInterval_);
	cf.writeEntry("submissionPaused", submissionPaused_);
}

void Configuration::read()
{
	Config cf(getConfigName(), Config::File);

	if (cf.readNumEntry("CONFIG_VERSION") != configVersion)
		return;

	cf.setGroup(CONFIGGROUPNAME);
	username_ = cf.readEntry("username", "");
	md5Password_ = cf.readEntry("password", "");
	cacheSaveInterval_ = cf.readNumEntry("cacheSaveInterval", 20);
	submissionPaused_ = cf.readBoolEntry("submissionPaused", false);
}
